// bitarray.cpp
// by David Lambert
// 2013-04-05

#include "bitarray.h"

using namespace std;

// Output Operator overload
ostream& operator<< (ostream& os, const BitArray& a){
   os << "(";

   for (int i = 0; i < a.Length(); i++)
      os << a.Query(i);

   os << ")";
}

// Equality Operator overload
bool operator== (const BitArray& a, const BitArray& b) {
   if (a.Length() != b.Length()) return false;
 
   for (int i = 0; i < a.arraySize; i++)
      if (a.barray[i] != b.barray[i]) return false;
 
   return true;
}

// Inequality Operator overload
bool operator!= (const BitArray& a, const BitArray& b) {
   return !(a == b);
}

// Default Constructor
BitArray::BitArray (unsigned int n) {
   arraySize = n / charSize;
   // array size has to be a multiple of the char type's size
   if (n % charSize != 0) arraySize += 1;

   barray = new unsigned char[arraySize];

   for (int i = 0; i < arraySize ; i++)
      barray[i] = '\0';
}

// Copy Constructor
BitArray::BitArray (const BitArray& ba) {
   barray = new unsigned char[ba.arraySize];
   arraySize = ba.arraySize;
   for (int i = 0; i < ba.arraySize; i++)
      barray[i] = ba.barray[i];
}

// Destructor
BitArray::~BitArray() {
   delete [] barray;
}

// Assignment Operator overload
BitArray& BitArray::operator= (const BitArray& a) {
   if (*this == a) return *this;   // don't copy the same BitArray to itself

   delete [] barray;
   arraySize = a.arraySize;
   barray = new unsigned char[arraySize];

   for (int i = 0; i < arraySize; i++)
      barray[i] = a.barray[i];

   return *this;
}

// Length function
// Returns total number of bits in the allocated array
unsigned int BitArray::Length () const {
   return arraySize * charSize;
}

// Set function
// Sets bit at index to 1
void BitArray::Set (unsigned int index) {
   barray[index / charSize] |= 1 << (charSize - (index % charSize)) - 1; 
}

// Unset function
// Sets bit at index to 0
void BitArray::Unset (unsigned int index) {
   barray[index / charSize] &= ~(1 << (charSize - (index % charSize)) - 1);
}

// Flip function
// Changes bit at index to its opposite
void BitArray::Flip (unsigned int index) {
   barray[index / charSize] ^= 1 << (charSize - (index % charSize)) - 1;
}

// Query function
// Returns true if bit at index is 1, otherwise returns false
bool BitArray::Query (unsigned int index) const {
   return barray[index / charSize] & 1 << (charSize - (index % charSize)) - 1;
}

